<?php

namespace App\Controllers;

use App\Core\Database;
use App\Services\AuthService;
use App\Services\LogService;
use App\Services\LanguageService;
use App\Utils\DarkMode;

/**
 * Contrôleur pour la gestion des certificats.
 * (Création, révocation, affichage).
 */
class CertificateController
{
    private $db;
    private $authService;
    private $logService;
    private $langService;

    /**
     * Constructeur du CertificateController.
     */
    public function __construct()
    {
        $this->db = Database::getInstance();
        $this->authService = new AuthService($this->db);
        $this->logService = new LogService(APP_LOG_PATH);
        $this->langService = new LanguageService(APP_ROOT_DIR . '/src/Lang/');
    }

    /**
     * Affiche la liste des certificats, regroupés par périmètre fonctionnel.
     */
    public function index()
    {
        if (!$this->authService->isLoggedIn()) {
            header('Location: /login');
            exit();
        }

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();
        $userRole = $this->authService->getUserRole();

        // Récupérer les périmètres et les certificats
        // Joindre pour obtenir le nom du périmètre
        $certificates = $this->db->query("
            SELECT
                c.id, c.name, c.type, c.expiration_date, c.is_revoked, c.revoked_at,
                fp.name as perimeter_name
            FROM
                certificates c
            LEFT JOIN
                functional_perimeters fp ON c.functional_perimeter_id = fp.id
            ORDER BY
                fp.name IS NULL DESC, fp.name ASC, c.type DESC, c.expiration_date DESC
        ")->fetchAll();

        // Regrouper les certificats par périmètre fonctionnel
        $groupedCertificates = [];
        foreach ($certificates as $cert) {
            $perimeterName = $cert['perimeter_name'] ?? 'Certificats Root'; // Nom pour le groupe Root
            if (!isset($groupedCertificates[$perimeterName])) {
                $groupedCertificates[$perimeterName] = [];
            }
            $groupedCertificates[$perimeterName][] = $cert;
        }

        $successMessage = $_SESSION['success'] ?? null;
        unset($_SESSION['success']);
        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/certificates/index.php';
    }

    /**
     * Affiche le formulaire de création d'un nouveau certificat.
     */
    public function showCreateForm()
    {
        if (!$this->authService->isLoggedIn()) {
            header('Location: /login');
            exit();
        }

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();

        // Récupérer la liste des périmètres fonctionnels pour le sélecteur
        $perimeters = $this->db->query("SELECT id, name FROM functional_perimeters ORDER BY name ASC")->fetchAll();

        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/certificates/create.php';
    }

    /**
     * Traite la soumission du formulaire de création de certificat.
     */
    public function create()
    {
        if (!$this->authService->isLoggedIn() || $_SERVER['REQUEST_METHOD'] !== 'POST') {
            header('Location: /login');
            exit();
        }

        $subdomainName = trim($_POST['subdomain_name'] ?? '');
        $functionalPerimeterId = $_POST['functional_perimeter_id'] ?? null;
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();

        if (empty($subdomainName) || empty($functionalPerimeterId)) {
            $_SESSION['error'] = $this->langService->__('cert_create_error_empty_fields');
            header('Location: /certificates/create');
            exit();
        }

        // Récupérer le nom du périmètre fonctionnel pour le script shell
        $stmt = $this->db->prepare("SELECT name FROM functional_perimeters WHERE id = ?");
        $stmt->execute([$functionalPerimeterId]);
        $perimeter = $stmt->fetch();

        if (!$perimeter) {
            $_SESSION['error'] = $this->langService->__('cert_create_error_perimeter_not_found');
            header('Location: /certificates/create');
            exit();
        }
        $functionalPerimeterName = $perimeter['name'];

        // Préparer la commande du script shell
        // Important: utiliser escapeshellarg pour protéger les arguments
        $command = escapeshellcmd(SCRIPTS_PATH . '/create_cert.sh') . ' ' .
                   escapeshellarg($subdomainName) . ' ' .
                   escapeshellarg($functionalPerimeterName);

        $this->logService->log('info', "Tentative de création du certificat '$subdomainName' pour le périmètre '$functionalPerimeterName'. Commande: '$command'", $userId, $ipAddress);

        // Exécuter le script shell
        $output = shell_exec($command . ' 2>&1'); // Redirige stderr vers stdout

        // Vérifier le résultat du script (simple vérification de chaîne, une meilleure parsage serait utile)
        if (strpos($output, "Certificat '${subdomainName}.${functionalPerimeterName}.cert' créé avec succès") !== false) {
            // Extraire la date d'expiration du certificat créé (en lisant le fichier cert ou en estimant 1 an)
            $certFileName = "{$subdomainName}.{$functionalPerimeterName}.cert.pem";
            $fullCertPath = INTERMEDIATE_CA_PATH_BASE . "/{$functionalPerimeterName}/certs/{$certFileName}";

            $expirationDate = (new \DateTime('+1 year'))->format('Y-m-d H:i:s'); // Valeur par défaut
            if (file_exists($fullCertPath)) {
                $certInfo = shell_exec("openssl x509 -in " . escapeshellarg($fullCertPath) . " -noout -enddate 2>/dev/null | cut -d= -f2");
                $expirationTimestamp = strtotime($certInfo);
                if ($expirationTimestamp) {
                    $expirationDate = date('Y-m-d H:i:s', $expirationTimestamp);
                }
            }


            // Enregistrer le certificat dans la base de données
            $stmt = $this->db->prepare("INSERT INTO certificates (name, type, functional_perimeter_id, expiration_date) VALUES (?, ?, ?, ?)");
            $stmt->execute([$certFileName, 'simple', $functionalPerimeterId, $expirationDate]);

            $this->logService->log('info', "Certificat '{$certFileName}' créé et enregistré pour le périmètre '{$functionalPerimeterName}'.", $userId, $ipAddress);
            $_SESSION['success'] = $this->langService->__('cert_create_success');
        } else {
            $_SESSION['error'] = $this->langService->__('cert_create_error', ['output' => htmlspecialchars($output)]);
            $this->logService->log('error', "Échec création certificat '$subdomainName' pour périmètre '$functionalPerimeterName'. Output: $output", $userId, $ipAddress);
        }

        header('Location: /certificates');
        exit();
    }

    /**
     * Traite la révocation d'un certificat.
     */
    public function revoke()
    {
        if (!$this->authService->isLoggedIn() || $_SERVER['REQUEST_METHOD'] !== 'POST') {
            header('Location: /login');
            exit();
        }

        $certificateId = $_POST['certificate_id'] ?? null;
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();

        if (empty($certificateId)) {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_id_missing');
            header('Location: /certificates');
            exit();
        }

        // Récupérer les informations du certificat depuis la DB
        $stmt = $this->db->prepare("SELECT c.name, c.type, fp.name as perimeter_name FROM certificates c LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id WHERE c.id = ?");
        $stmt->execute([$certificateId]);
        $cert = $stmt->fetch();

        if (!$cert) {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_not_found');
            header('Location: /certificates');
            exit();
        }

        // Empêcher la révocation des certificats Root ou Intermédiaires via l'interface
        if ($cert['type'] === 'root' || $cert['type'] === 'intermediate') {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_ca_revocation');
            header('Location: /certificates');
            exit();
        }

        // Préparer le nom de base du certificat pour le script (sans l'extension .pem)
        $certBaseName = str_replace('.cert.pem', '.cert', $cert['name']);
        $functionalPerimeterName = $cert['perimeter_name'];

        // Vérifier si le certificat n'est pas déjà révoqué dans la DB
        if ($cert['is_revoked']) {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_already_revoked');
            header('Location: /certificates');
            exit();
        }

        // Appeler le script shell de révocation
        $command = escapeshellcmd(SCRIPTS_PATH . '/revoke_cert.sh') . ' ' .
                   escapeshellarg($certBaseName) . ' ' .
                   escapeshellarg($functionalPerimeterName);

        $this->logService->log('info', "Tentative de révocation du certificat '{$cert['name']}' pour le périmètre '$functionalPerimeterName'. Commande: '$command'", $userId, $ipAddress);

        $output = shell_exec($command . ' 2>&1');

        if (strpos($output, "Certificat '$certBaseName' révoqué avec succès.") !== false) {
            // Mettre à jour le statut du certificat dans la base de données
            $stmt = $this->db->prepare("UPDATE certificates SET is_revoked = TRUE, revoked_at = NOW() WHERE id = ?");
            $stmt->execute([$certificateId]);

            $this->logService->log('info', "Certificat '{$cert['name']}' révoqué et enregistré en DB.", $userId, $ipAddress);
            $_SESSION['success'] = $this->langService->__('cert_revoke_success');
        } else {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error', ['output' => htmlspecialchars($output)]);
            $this->logService->log('error', "Échec révocation certificat '{$cert['name']}': $output", $userId, $ipAddress);
        }

        header('Location: /certificates');
        exit();
    }
}
